@@ -4,6 +4,7 @@
 #include "config.h"
 #endif
 
+#define _GNU_SOURCE /* for strsignal() */
 #include <unistd.h>
 #include <errno.h>
 #include <stdio.h>
@@ -239,7 +240,7 @@ static int old_lockfile = 1;
 /* Ensure that the lock is released if we are interrupted.  */
 static void
 handler (int sig) {
-     die (EX_USER, "%s", sys_siglist[sig]);
+     die (EX_USER, "%s", strsignal(sig));
 }
 
 static void
