@@ -1,9 +1,12 @@
-/* $Header: /cvsroot/watchdog/watchdog/src/shutdown.c,v 1.3 2007/02/12 09:42:07 meskes Exp $ */
+/* $Header: /cvsroot/watchdog/watchdog/src/shutdown.c,v 1.5 2009/02/11 14:01:05 meskes Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
+#define _XOPEN_SOURCE 500      /* for getsid(2) */
+#define _BSD_SOURCE            /* for acct(2) */
+
 #include <dirent.h>
 #include <errno.h>
 #include <fcntl.h>
@@ -18,6 +21,10 @@
 #include <sys/mman.h>
 #include <sys/param.h>
 #include <sys/stat.h>
+#include <sys/types.h>
+#include <unistd.h>
+#include <time.h>
+#include <rpc/types.h>         /* for caddr_t */
 
 #include "watch_err.h"
 #include "extern.h"
@@ -51,6 +58,8 @@ static struct mntent rootfs;
 #if defined(_POSIX_MEMLOCK)
 extern int mlocked, realtime;
 #endif /* _POSIX_MEMLOCK */
+extern volatile sig_atomic_t _running;
+extern int devtimeout; /* From watchdog.c */
 
 jmp_buf ret2dog;
 
@@ -78,7 +87,7 @@ static void log_end()
 static void close_all()
 {
     if (watchdog != -1) {
-        if ( write(watchdog, "V", 1) < 0 ) {
+        if (write(watchdog, "V", 1) < 0 ) {
 		int err = errno;
 #if USE_SYSLOG
 		syslog(LOG_ERR, "write watchdog device gave error %d = '%m'!", err);
@@ -129,8 +138,15 @@ static void close_all()
     }
 }
 
+
+void sigterm_handler(int arg)
+{
+    _running = 0;
+}
+
+
 /* on exit we close the device and log that we stop */
-void terminate(int arg)
+void terminate(void)
 {
 #if defined(_POSIX_MEMLOCK)
     if (realtime == TRUE && mlocked == TRUE)
@@ -157,10 +173,10 @@ static void panic(void)
 {
     /* if we are still alive, we just exit */
     close_all();
-    fprintf(stderr, "WATCHDOG PANIC: Still alive after sleeping %d seconds!\n", 4 * TIMER_MARGIN);
+    fprintf(stderr, "WATCHDOG PANIC: Still alive after sleeping %d seconds!\n", 4 * devtimeout);
 #if USE_SYSLOG
     openlog(progname, LOG_PID, LOG_DAEMON);
-    syslog(LOG_ALERT, "still alive after sleeping %d seconds", 4 * TIMER_MARGIN);
+    syslog(LOG_ALERT, "still alive after sleeping %d seconds", 4 * devtimeout);
     closelog();
 #endif
     exit(1);
@@ -172,7 +188,8 @@ static void mnt_off()
     struct mntent *mnt;
 
     fp = setmntent(MNTTAB, "r");
-    while ((mnt = getmntent(fp)) != (struct mntent *) 0) {
+    /* in some rare cases fp might be NULL so be careful */
+    while (fp != NULL && ((mnt = getmntent(fp)) != (struct mntent *) 0)) {
 	/* First check if swap */
 	if (!strcmp(mnt->mnt_type, MNTTYPE_SWAP))
 	    if (swapoff(mnt->mnt_fsname) < 0)
@@ -322,62 +339,73 @@ void do_shutdown(int errorcode)
     char *seedbck = RANDOM_SEED;
 
     /* soft-boot the system */
-    /* first close the open files */
-    close_all();
+    /* do not close open files here, they will be closed later anyway */
+    /* close_all(); */
 
     /* if we will halt the system we should try to tell a sysadmin */
     if (admin != NULL) {
 	/* send mail to the system admin */
 	FILE *ph;
 	char exe[128];
+        struct stat buf;
 
-	/* only can send an email if sendmail binary exists so make shell check
-	 * that first, or else we will get a broken pipe in pclose */
-	sprintf(exe, "[ -x %s ] && %s -i %s", PATH_SENDMAIL, PATH_SENDMAIL, admin);
-	ph = popen(exe, "w");
-	if (ph == NULL) {
+	/* Only can send an email if sendmail binary exists so check
+	 * that first, or else we will get a broken pipe in pclose.
+	 * We cannot let the shell check, because a non-existant or
+	 * non-executable sendmail binary means that the pipe is closed faster
+	 * than we can write to it. */
+	if ((stat(PATH_SENDMAIL, &buf) != 0) || ((buf.st_mode&S_IXUSR) == 0))
 #if USE_SYSLOG
-	    syslog(LOG_ERR, "cannot start %s (errno = %d)", PATH_SENDMAIL, errno);
+		syslog(LOG_ERR, "%s does not exist or is not executable (errno = %d)", PATH_SENDMAIL, errno);
 #endif				/* USE_SYSLOG */
-	} else {
-	    char myname[MAXHOSTNAMELEN + 1];
-	    struct hostent *hp;
+	else
+	{
+		sprintf(exe, "%s -i %s", PATH_SENDMAIL, admin);
+		ph = popen(exe, "w");
+		if (ph == NULL) {
+#if USE_SYSLOG
+		    syslog(LOG_ERR, "cannot start %s (errno = %d)", PATH_SENDMAIL, errno);
+#endif				/* USE_SYSLOG */
+		} else {
+		    char myname[MAXHOSTNAMELEN + 1];
+		    struct hostent *hp;
 
-	    /* get my name */
-	    gethostname(myname, sizeof(myname));
+		    /* get my name */
+		    gethostname(myname, sizeof(myname));
 
-	    fprintf(ph, "To: %s\n", admin);
-	    if (ferror(ph) != 0) {
+		    fprintf(ph, "To: %s\n", admin);
+		    if (ferror(ph) != 0) {
 #if USE_SYSLOG
-		syslog(LOG_ERR, "cannot send mail (errno = %d)", errno);
+			syslog(LOG_ERR, "cannot send mail (errno = %d)", errno);
 #endif				/* USE_SYSLOG */
-	    }
-	    /* if possible use the full name including domain */
-	    if ((hp = gethostbyname(myname)) != NULL)
-		fprintf(ph, "Subject: %s is going down!\n\n", hp->h_name);
-	    else
-		fprintf(ph, "Subject: %s is going down!\n\n", myname);
-	    if (ferror(ph) != 0) {
+		    }
+		    /* if possible use the full name including domain */
+		    if ((hp = gethostbyname(myname)) != NULL)
+			fprintf(ph, "Subject: %s is going down!\n\n", hp->h_name);
+		    else
+			fprintf(ph, "Subject: %s is going down!\n\n", myname);
+		    if (ferror(ph) != 0) {
 #if USE_SYSLOG
-		syslog(LOG_ERR, "cannot send mail (errno = %d)", errno);
+			syslog(LOG_ERR, "cannot send mail (errno = %d)", errno);
 #endif				/* USE_SYSLOG */
-	    }
+		    }
 
-	    if (errorcode == ETOOHOT)
-		fprintf(ph, "Message from watchdog:\nIt is too hot to keep on working. The system will be halted!\n");
-	    else
-	    	fprintf(ph, "Message from watchdog:\nThe system will be rebooted because of error %d!\n", errorcode);
-	    if (ferror(ph) != 0) {
+		    if (errorcode == ETOOHOT)
+			fprintf(ph, "Message from watchdog:\nIt is too hot to keep on working. The system will be halted!\n");
+		    else
+			fprintf(ph, "Message from watchdog:\nThe system will be rebooted because of error %d!\n", errorcode);
+		    if (ferror(ph) != 0) {
 #if USE_SYSLOG
-		syslog(LOG_ERR, "cannot send mail (errno = %d)", errno);
+			syslog(LOG_ERR, "cannot send mail (errno = %d)", errno);
 #endif				/* USE_SYSLOG */
-	    }
-	    if (pclose(ph) == -1) {
+		    }
+		    if (pclose(ph) == -1) {
 #if USE_SYSLOG
-		syslog(LOG_ERR, "cannot finish mail (errno = %d)", errno);
+			syslog(LOG_ERR, "cannot finish mail (errno = %d)", errno);
 #endif				/* USE_SYSLOG */
-	    }
-	    /* finally give the system a little bit of time to deliver */
+		    }
+		    /* finally give the system a little bit of time to deliver */
+		}
 	}
     }
 
@@ -387,19 +415,22 @@ void do_shutdown(int errorcode)
     closelog();
 #endif				/* USE_SYSLOG */
 
+    keep_alive();
     sleep(10);			/* make sure log is written and mail is send */
+    keep_alive();
 
     /* We cannot start shutdown, since init might not be able to fork. */
     /* That would stop the reboot process. So we try rebooting the system */
     /* ourselves. Note, that it is very likely we cannot start any rc */
     /* script either, so we do it all here. */
 
-    /* Start with closing the files. */
+    /* Close all files except the watchdog device. */
     for (i = 0; i < 3; i++)
 	if (!isatty(i))
 	    close(i);
     for (i = 3; i < 20; i++)
-	close(i);
+	if (i != watchdog)
+		close(i);
     close(255);
 
     /* Ignore all signals. */
@@ -413,6 +444,7 @@ void do_shutdown(int errorcode)
     (void) killall5(SIGTERM);
     sleep(5);
     (void) killall5(SIGKILL);
+    keep_alive();
 
     /* Record the fact that we're going down */
     if ((fd = open(_PATH_WTMP, O_WRONLY | O_APPEND)) >= 0) {
@@ -453,6 +485,8 @@ void do_shutdown(int errorcode)
     if (acct(NULL) < 0)
 	perror(progname);
 
+    keep_alive();
+
     /* Turn off quota and swap */
     mnt_off();
 
@@ -498,13 +532,7 @@ void do_shutdown(int errorcode)
     /* okay we should never reach this point, */
     /* but if we do we will cause the hard reset */
 
-    /* open the device again */
-    /* don't check for error, it won't help anyway here */
-    if (devname != NULL) {
-	open(devname, O_WRONLY);
-
-	sleep(TIMER_MARGIN * 4);
-    }
+    sleep(devtimeout * 4);
 
     /* unbelievable: we're still alive */
     panic();
