@@ -25,13 +25,12 @@
 #include <sys/wait.h>
 #include <sys/stat.h>
 #include <sys/types.h>
+#include <sys/ioctl.h>
 #include <linux/watchdog.h>
-#define __USE_GNU
 #include <string.h>
 
-#if !defined(__GLIBC__)		/* __GLIBC__ */
-extern char *basename(const char *);
-#endif				/* __GLIBC__ */
+#include <sys/ioctl.h>
+#include <libgen.h>
 
 #include <unistd.h>
 
@@ -248,14 +247,15 @@ static void read_config(char *filename, char *progname)
     int gotload5 = FALSE, gotload15 = FALSE;
 
     if ((wc = fopen(filename, "r")) == NULL) {
-	perror(progname);
+	fprintf(stderr, "%s: Can't open config file \"%s\": %s ", progname, filename, strerror(errno));
 	exit(1);
     }
 
     while (!feof(wc)) {
-	char line[CONFIG_LINE_LEN];
+	char *line = NULL;
+	size_t n;
 
-	if (fgets(line, CONFIG_LINE_LEN, wc) == NULL) {
+	if (getline(&line, &n, wc) == -1) {
 	    if (!ferror(wc))
 		break;
 	    else {
@@ -570,7 +570,8 @@ int main(int argc, char *const argv[])
 		(void) fprintf(stderr, "%s: unknown protocol icmp.\n", progname);
 		exit(1);
 	    }
-	    if ((net->sock_fp = socket(AF_INET, SOCK_RAW, proto->p_proto)) < 0) {
+	    if ((net->sock_fp = socket(AF_INET, SOCK_RAW, proto->p_proto)) < 0
+		|| fcntl(net->sock_fp, F_SETFD, 1)) {
 		perror(progname);
 		exit(1);
 	    }
@@ -690,7 +691,7 @@ int main(int argc, char *const argv[])
 	       driver default) */
 	    if (ioctl(watchdog, WDIOC_SETTIMEOUT, &devtimeout) < 0) {
 #if USE_SYSLOG
-            	syslog(LOG_ERR, "cannot set timeout %s (errno = %d = '%m')", strerror(errno), devtimeout, errno);
+                syslog(LOG_ERR, "cannot set timeout %d (errno = %d = '%m')", devtimeout, errno);
 #else				
             	perror(progname);
 #endif			   
