@@ -24,7 +24,7 @@
 #include <stdlib.h>
 #include <sys/mman.h>
 #include <sys/wait.h>
-#define __USE_GNU
+#include <libgen.h>
 #include <string.h>
 #include <syslog.h>
 
@@ -138,9 +138,10 @@ static void read_config(char *filename, char *progname)
     }
 
     while ( !feof(wc) ) {
-        char line[CONFIG_LINE_LEN];
+	char *line = NULL;
+	size_t n;
 
-        if ( fgets(line, CONFIG_LINE_LEN, wc) == NULL ) {
+	if (getline(&line, &n, wc) == -1) {
             if ( !ferror(wc) )
                 break;
             else {
